/*
 * Copyright 2012 Adaptrex, LLC
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *    http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package com.adaptrex.core.config;

import com.fasterxml.jackson.databind.ObjectMapper;
import java.io.InputStream;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import javax.servlet.ServletContext;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

public class AdaptrexConfig {

    private static volatile Map<String, Object> map = new HashMap<String, Object>();
    private static Logger log = LoggerFactory.getLogger(AdaptrexConfig.class);

    /*
     * Some sugar to make it easier to read the config
     */
    private static boolean debug = false;
    
    public static void init(ServletContext context) {
        ObjectMapper mapper = new ObjectMapper();
        
        /*
         * Add the webapp specific config
         */
        InputStream contextFile = context.getResourceAsStream("/WEB-INF/adaptrex.webapp.config");
        try {
            map.putAll(mapper.readValue(contextFile, Map.class));
        } catch (Exception e) {
            log.info("Could not load adaptrex.webapp.config for '" + context.getContextPath() + "'");
        }

        try {
            ClassLoader adaptrexClassloader = Thread.currentThread().getContextClassLoader().getParent();
            InputStream adaptrexFile = adaptrexClassloader.getResourceAsStream("adaptrex.config");
            
            Map<String,Object> adaptrexConfigMap = mapper.readValue(adaptrexFile, Map.class);

            /*
             * Add the global adaptrex config.  We don't need info on the other
             * webapps, so remove the webapps item.
             */
            map.putAll(adaptrexConfigMap);
            map.remove("webapps");
            
            /*
             * Add the webapp config from the global adaptrex config
             */
            for (Map<String, Object> contextConfig : (List<Map<String, Object>>) adaptrexConfigMap.get("webapps")) {
                String path = (String) contextConfig.get("contextPath");
                if (path == null) {
		    path = "";
		}
                if (context.getContextPath().replace("/", "").equals(path)) {
                    map.putAll(contextConfig);
                    break;
                }
            }
        } catch (Exception e) {
            log.warn("Could not load adaptrex.config", e);
        }
        
        
        /*
         * Update our sugar fields
         */
        debug = map.get("debug") != null && (Boolean) map.get("debug");
    }

    
    public static boolean isDebug() {
        return debug;
    }

    public static Object get(String key) {
        return map.get(key);
    }
}
