/*
 * Copyright 2012 Adaptrex, LLC
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *    http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package com.adaptrex.core.config;

import com.adaptrex.core.Adaptrex;
import com.adaptrex.core.persistence.jpa.JpaPersistence;
import javax.servlet.ServletContextEvent;
import javax.servlet.ServletContextListener;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

/**
 * Initialize Adaptrex on webapp launch.  Adaptrex global configurations are added to the context.
 * @author adaptrex
 */
public class ConfigListener implements ServletContextListener {
	
	private Logger log = LoggerFactory.getLogger(ConfigListener.class);
	
	@Override
	public void contextInitialized(ServletContextEvent event) {
            /*
             * Create the configuration
             */
            AdaptrexConfig.init(event.getServletContext());
            
            /*
             * Initialize Persistence
             */
            String pimplName = (String) AdaptrexConfig.get("persistence");
            
            try {
                if (pimplName == null || pimplName.isEmpty()) {
                    Adaptrex.initPersistence(JpaPersistence.class);
                } else {
                    Adaptrex.initPersistence(Class.forName(pimplName));
                }
            } catch (InstantiationException ex) {
                log.warn(null, ex);
            } catch (IllegalAccessException ex) {
                log.warn(null, ex);
            } catch (ClassNotFoundException ex) {
                log.warn(null, ex);
            }
	}

    @Override
    public void contextDestroyed(ServletContextEvent sce) {
        Adaptrex.destroyPersistence();
    }
}
